//=============================================================================
// (c) Copyright 2009 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: DSCDAAutoCal.c   v1.00
//
// Depedency :	Universal Driver version 6.02
//				DSCUD.H - Include file.
//				DSCUDBCL.LIB - Library file for MS-DOS
//						( USE BC 5.02 compiler )
//				LIBDSCUD_6.02_1.a	- Library file for Linux 2.6.23
//						( USE GCC 3.xx compiler )
//				DSCUD.LIB	- Static library file for Windows XP
//						( USE VC++ 6.0 or Visual Studio 2005 )
//				DSCUD_API.LIB	- Static library for Windows CE 6.0
//						( Use Visual Studio 2005 with Platform Builder 6.0 )
// Description: This sample code demonstrates the usage of UD function
//				dscDAAutoCal(). This sample code describes the usage
//				for Athena-II SBC. AutoCalibration will use the onboard
//              hardware to calibrate the D/A circuitry on the board,
//              then verify that the process results fall within tolerances.
//              The calibration values will be stored in the EEPROM,
//              along with one set of values to be loaded at boot time.
// ************************* History ********************
// Version		Engineer		Date		Description
// 1.0			James Moore		07/27/09	Created.
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif
#include <windows.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
// diamond driver includes
#include "dscud.h"
#endif

// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
#include <mem.h>
// diamond driver includes
#include "dscud.h"
#endif

// Linux
#if defined(linux)
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>
// diamond driver includes
#include "dscud.h"

#include <sys/time.h>

#endif

#define DA_AUTOCAL_TOLERANCE 2

// var declarations
BYTE result;  // returned error code
DSCB dscb;    // handle used to refer to the board
DSCCB dsccb;  // structure containing board settings
DSCDACALPARAMS dscdacalparams;// structure containing auto-calibration settings
ERRPARAMS errorParams;        // structure stores and returns error code
int intBuff;      // temp variables of size int
float floatBuff;  // temp variables of size float
int i;	          // miscellaneous counter

char input_buffer[20]; // Buffer for parsing user character input.

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. AutoCal Settings Initialization
//	    IV. Auto-Calibration
//		 V. Calibration Verification
//	    VI. Cleanup
//
//=============================================================================

int main( void )
{
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. initialize the driver, using the driver version for validation
	//=========================================================================

	if( ( result = dscInit( DSC_VERSION ) ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the AthenaII board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//=========================================================================

	printf ( "\nATHENAII BOARD INITIALIZATION:\n" );

	dsccb.base_address = 0x280;

	dsccb.int_level = 5;

	if ( dscInitBoard ( DSC_ATHENAII, &dsccb, &dscb ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	//III. AUTOCAL SETTINGS INITIALIZATION
	//
	//		Initialize the structure containing the DA autocal settings. The
	//		structure element, dscautocal, contains a decimal value ranging from
	//		0-10.0, that represents the  D/A programmable voltage to be calibrated
	//		for. For example, calibrating for D/A ouptut of 6.8V, set
	//		dscdacalparams.darange = 6.8.
	//
	//		STEPS TO FOLLOW:
	//
	//		1. set the DA range code
	//=========================================================================

	printf ( "\nAUTOCAL SETTINGS INITIALIZATION:\n" );

	// Recommended value:
    //  dscautocal.darange = 10.0f;

	memset(&dscdacalparams, 0, sizeof(DSCDACALPARAMS));

	printf ( "Enter the DA range code 0 being fixed \nor a float between 0-10.0 being the voltage (default: 10.0): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	dscdacalparams.darange = 10.0;
	}
	else
	{
        sscanf ( input_buffer, "%f", &floatBuff );
    	dscdacalparams.darange = (FLOAT)floatBuff;
	}

	//=========================================================================
	// IV. AUTO-CALIBRATION
	//
	//	   Perform the actual DA auto-calibration of the board.
	//
	//	   STEPS TO FOLLOW:
	//
	//	   1. perform auto-calibration
	//=========================================================================

	printf ( "\nAUTO-CALIBRATION:\n" );
	printf ( "Starting DA auto-calibration...\n" );

	if( ( result = dscDAAutoCal ( dscb, &dscdacalparams ) ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscDAAutoCal error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// V. CALIBRATION VERIFICATION
	//
	//	  To verify that the auto-calibration was successfully completed, you
	//	  should run dscDACalVerify() to ensure that the gain and offset errors
	//	  meet a specified tolerance (2 in this example).
	//
	//	  STEPS TO FOLLOW:
	//
	//	  1. verify that the calibration was successful and output results
	//=========================================================================

	printf ( "\nCALIBRATION VERIFICATION:\n" );

	if( ( result = dscDACalVerify ( dscb, &dscdacalparams ) ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscDACalVerify error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	printf ( "Offset Error: %9.3f, Gain Error: %9.3f\n", dscdacalparams.offset, dscdacalparams.gain );

	if ( fabs( dscdacalparams.offset ) > DA_AUTOCAL_TOLERANCE ||
		 fabs( dscdacalparams.gain ) > DA_AUTOCAL_TOLERANCE )
		printf ( "Value for offset or gain exceeded specified tolerance\n" );
	else printf ( "Values for offset and gain met specified tolerance\n" );

	//=========================================================================
	// VI. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree();

	printf ( "\nDSCDAAutoCal completed. \n" );

	return 0;
} // end main()



#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif