//=============================================================================
// (c) Copyright 2009 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: DSCDAConvert.c   v1.00
//
// Depedency :	Universal Driver version 6.02
//				DSCUD.H - Include file.
//				DSCUDBCL.LIB - Library file for MS-DOS
//						( USE BC 5.02 compiler )
//				LIBDSCUD_6.02_1.a	- Library file for Linux 2.6.23
//						( USE GCC 3.xx compiler )
//				DSCUD.LIB	- Static library file for Windows XP
//						( USE VC++ 6.0 or Visual Studio 2005 )
//				DSCUD_API.LIB	- Static library for Windows CE 6.0
//						( Use Visual Studio 2005 with Platform Builder 6.0 )
// Description: This sample code demonstrates the usage of UD function
//				dscDAConvert(). This sample code describes the usage
//				for Athena-II SBC. Channel and 12-bit output code are prompted
//              from the user, then output to the board. Other settings are
//              controlled via the jumpers for this board.
// ************************* History ********************
// Version		Engineer		Date		Description
// 1.0			James Moore		07/29/09	Created.
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif

#include <windows.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// Linux
#if defined(linux)
#include <stdlib.h>
#include <math.h>
#include <time.h>
// diamond driver includes
#include "dscud.h"

#include <sys/time.h>

#endif

// macros defined
#define ERROR_PREFIX "ATHENA II Driver ERROR:"

// var declarations
BYTE result;  // returned error code
DSCB dscb;    // handle used to refer to the board
DSCCB dsccb;  // structure containing board settings
DSCDACODE output_code;  // DA code for desired output voltage
ERRPARAMS errorParams;  // structure for returning error code and error string
int intBuff;            // temp variables of size int
int i;        // miscellaneous counter

char input_buffer[20];
int polarity = 0;
int gain = 0;
int load_cal = 0;
int dasim = 0;
int channel; // selected output channel

DSCDASETTINGS dasettings;


//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//     STEPS TO FOLLOW:
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. DA Conversion and Output
//	    IV. Cleanup
//
//=============================================================================

int main()
{
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. initialize the driver, using the driver version for validation
	//=========================================================================

	if ( dscInit( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the ATHENA II board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. set the board type to DSC_ATHENAII for ATHENA II board
	//	   2. set the base address to 0x280
	//	   3. set the interrupt level 5
	//	   4. intialize and register the board with the driver, after which
	//		  the struct, dscb, now holds the handle for the board
	//=========================================================================

	printf ( "\nATHENA II BOARD INITIALIZATION:\n" );

	dsccb.base_address = 0x280;

	dsccb.int_level = 5;

	if ( dscInitBoard ( DSC_ATHENAII, &dsccb, &dscb ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// III. DA CONVERSION AND OUTPUT
	//
	//	    Perform the actual DA conversion, then take an AD sample to verify
	//	    and output the results. Note that in order to verify the conversion
	//	    process on a particular output channel, we must set the input
	//	    channel on the AD conversion to (Output Channel + 4). To calculate
	//	    the output code to obtain a particular input voltage, we must use
	//	    one of the formulas located in the manual for your board (under the
	//	    the section, "Generating an Analog Output"). For example, the
	//	    formula for bipolar mode is:
	//
	//	    Output code = Output Voltage / Full-Scale Voltage * 2048 + 2048
	//
	//	    To calculate the actual input voltages from sample codes, we must
	//	    convert the sample code (which must be cast to a short to get the
	//	    correct code) and then plug it into one of the formulas located in
	//	    the manual for your board (under "A/D Conversion Formulas"). For
	//	    example, if you are using a bipolar input range and 32-bit signed
	//	    integers:
	//
	//	    Input voltage = (AD Code / 32768) x Full-Scale Voltage
	//
	//	    STEPS TO FOLLOW:
	//
	//	    1. set the channel (must be between 0-3)
	//	    2. set the output code
	//	    3. write the output value to the selected output channel
	//	    4. repeat step 3 until a key is pressed
	//=========================================================================

	do
	{
		printf ( "\nDA CONVERSION AND OUTPUT\n" );

		printf ( "Enter the channel (0-3; default: 0): " );
	    fgets ( input_buffer, 20, stdin );
	    if ( input_buffer[0] == '\n' )
	    {
            channel = 0;
	    }
	    else
	    {
            sscanf ( input_buffer, "%d", &intBuff );
            channel = (BYTE)intBuff;
	    }

		printf ( "Enter the output code (0-4095; default: 2048): " );
	    fgets ( input_buffer, 20, stdin );
	    if ( input_buffer[0] == '\n' )
	    {
            output_code = 2048;
	    }
	    else
	    {
            sscanf ( input_buffer, "%d", &intBuff );
            output_code = (DSCDACODE)intBuff;
	    }


		printf ( "\nSending output code, %d, to channel %d...\n", (int) output_code, (int)channel );

		if ( ( result = dscDAConvert( dscb, channel, output_code ) ) != DE_NONE )
		{
			dscGetLastError(&errorParams);
			fprintf ( stderr, "dscDAConvert error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
			return 0;
		}

		printf ("Press 'q' to quit or ENTER to continue ");

	    fgets ( input_buffer, 20, stdin );

    } while ( input_buffer[0] != 'q' );

	//=========================================================================
	// IV. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree();

	printf ( "\nDSCDAConvert completed.\n" );

	return 0;
} // end main()


#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
