//=============================================================================
// (c) Copyright 2009 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: DSCDIOFunctions.c   v1.00
//
// Depedency :	Universal Driver version 6.02
//				DSCUD.H - Include file.
//				DSCUDBCL.LIB - Library file for MS-DOS
//						( USE BC 5.02 compiler )
//				LIBDSCUD_6.02_1.a	- Library file for Linux 2.6.23
//						( USE GCC 3.xx compiler )
//				DSCUD.LIB	- Static library file for Windows XP
//						( USE VC++ 6.0 or Visual Studio 2005 )
//				DSCUD_API.LIB	- Static library for Windows CE 6.0
//						( Use Visual Studio 2005 with Platform Builder 6.0 )
// Description: This sample code demonstrates the usage of UD functions
//				dscDIOSetConfig(), dscDIOInputByte(), and dscDIOOutputByte().
//              This sample code describes the usage for Athena-II SBC.
//              User may select direction bits for DIO ports, output from
//              one port and inputting from another. The default behaviour
//              for the default options is for DIO port A to be fed back in
//              via a cable or wires to DIO port B. See the manual for pinouts.
// ************************* History ********************
// Version		Engineer		Date		Description
// 1.0			James Moore		07/31/09	Created.
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif

#include <windows.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// Linux
#if defined(linux)
#include <stdlib.h>
#include <math.h>
#include <time.h>
// diamond driver includes
#include "dscud.h"

#include <sys/time.h>

#endif

int DIOConfigurePort();
int DIOConfigurePorts();
int Output();
int Input();

// var declarations
BYTE result; // returned error code
DSCB dscb;   // handle used to refer to the board
DSCCB dsccb; // structure containing board settings
ERRPARAMS errorParams;  // structure for returning error code and error string
int intBuff;            // temp variables of size int
BYTE config_byte;      // DIO Configuration byte
BYTE output_b;          // output byte
BYTE input_b;           // input byte
BYTE port;               // digital I/O port
int menu;
char input_buffer[20];
int i; // miscellaneous counter

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. DA Conversion
//	    IV. Cleanup
//
//=============================================================================

int main()
{
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. initialize the driver, using the driver version for validation
	//=========================================================================

	if( dscInit( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the ATHENA II board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. set the board type to DSC_ATHENAII for ATHENA II board
	//	   2. set the base address to 0x280
	//	   3. set the interrupt level 5
	//	   4. intialize and register the board with the driver, after which
	//		  the struct, dscb, now holds the handle for the board
	//=========================================================================

	printf ( "\nATHENA II BOARD INITIALIZATION:\n" );

	// The Athena II has a set base address of 0x280 .
	dsccb.base_address = 0x280;

	dsccb.int_level = 5;


	if(dscInitBoard(DSC_ATHENAII, &dsccb, &dscb)!= DE_NONE)
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// III. DIO INPUT AND OUTPUT
	//
	//	Performs a set of byte-level inputs and outputs. The user is
	//	presented with a menu; once the ports are configured, the
	//	user may take input or give output to the DIO ports. A simple
	//	test is to feed the output of a DIO port to another port set
	//	for input.

	//	For word and bit-level operations, the steps are nearly
	//	identical with a few minor changes to function names and the size
	//	of the data to be transferred.
	//
	//	STEPS TO FOLLOW:
	//
	//	1. Call dscDIOSetConfig to setup the DIO ports input/output modes.
	//	2. Select a port for output.
	//	3. Enter the output byte value.
	//	4. Select a port for input.
	//=========================================================================

	printf ( "\nDIO INPUT AND OUTPUT:\n" );

	/*
	Example:
		config_byte = 0x82;
	*/

	do
	{


	printf ( "\n0.) Quit\n1.) Configure ports\n2.) Output\n3.) Input\n" );

	printf ( "Enter your choice: " );

        fgets ( input_buffer, 20, stdin );
	sscanf ( input_buffer, "%d", &menu );

	if ( menu == 1 )
	{
	  DIOConfigurePorts();
	}
	if ( menu == 2 )
	{
	  Output();
	}
	if ( menu == 3 )
	{
	  Input();
	}

	} while ( menu != 0 );



	//=========================================================================
	// IV. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree();

	printf ( "\nDSCDIOFunctions completed.\n" );

	return 0;
} // end main()


int DIOConfigurePorts()
{

	printf ( "PORT CONFIGURATION SECTION: \n" );

	port = 0;

    // See Athena II manual on page 62; Digital I/O control word, BASE+11

    // For example: port A pins can be routed into port B pins, using the configuration byte
    //   of 0x82  ( 0b10000010 ) for:
    //   DIOCTR | -- | -- | DIRA | DIRCH | -- | DIRB | DIRCL
    // Or another example; say you instead wanted to route the lower nibble of port C to
    //   the upper nibble of port C ( CL to CH ); you could use this configuration byte:
    //   0x88 ( 0b10001000 )

    printf ( "For Athena II DIO, port A pins can be manually wired into\nport B pins, using the configuration byte 0x82 (A = output; B = input).\n\n" );

	printf ( "Enter the configuration byte for\nports A, B, C in HEX (see manual; default: 0x82): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	config_byte = 0x82; // A is output; B is input
	}
	else
	{
        sscanf ( input_buffer, "%x", &intBuff );
    	config_byte = (BYTE) intBuff;
	}

	DIOConfigurePort();

	return 0;

}

// Configure ports with data already collected
int DIOConfigurePort()
{
	printf ( "Using Athena II onboard DIO ports: config_byte: [%x]\n", config_byte );

	if ( ( result = dscDIOSetConfig ( dscb, &config_byte ) ) != DE_NONE ) //set DIO configuration
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscDIOSetConfig error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		printf ( "Error using standard DIO ports.\n" );
		return 0;
	}

	return 0;
}

int Output()
{

	printf ( "\nOUTPUT section:\n" );

	printf ( "Enter port number (0-2; default: 0): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	port = 0;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	port = (BYTE) intBuff;
	}

	printf ( "Enter the value to port in HEX (default: 0xA5): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	output_b = 0xA5;
	}
	else
	{
        sscanf ( input_buffer, "%hx", &intBuff );
    	output_b = (BYTE) intBuff;
	}

	if ( ( result = dscDIOOutputByte ( dscb, port, output_b ) ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscDIOOutputByte error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	printf ( "Byte value sent to port: %x\n", (int)output_b );

	return 0;

}

int Input()
{
	printf ( "\nINPUT section: \n" );

	printf ( "Enter port number (0-2; default: 1): " );
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
    	port = 1;
	}
	else
	{
        sscanf ( input_buffer, "%d", &intBuff );
    	port = (BYTE) intBuff;
	}

	if ( ( result = dscDIOInputByte ( dscb, port, &input_b ) ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscDIOInputByte error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	printf ( "Byte value (previously) sent to port: %x, Byte value received from port: %x\n", (int)output_b, (int)input_b );

	return 0;
}


#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
